"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const fetch_1 = require("../resolve/fetch");
const pnpm_logger_1 = require("pnpm-logger");
const spawn = require("cross-spawn");
const gitLogger = pnpm_logger_1.default('git');
function fetchRes(res, target, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (res.tarball) {
            return fetch_1.fetchFromRemoteTarball(target, {
                shasum: res.shasum,
                tarball: res.tarball
            }, opts);
        }
        if (res.repo && res.ref) {
            return clone(res.repo, res.ref, target);
        }
        if (res.fetch) {
            return res.fetch(target);
        }
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = fetchRes;
/**
 * clone a git repository.
 */
function clone(repo, ref, dest) {
    return __awaiter(this, void 0, void 0, function* () {
        yield new Promise((resolve, reject) => {
            const args = ['clone', '-b', ref, repo, dest, '--single-branch'];
            gitLogger.debug(`cloning git repository from ${repo}`);
            const git = spawnGit(args);
            let errMsg = '';
            git.stderr.on('data', (data) => errMsg += data);
            git.on('close', (code) => (code ? errorHandler() : resolve()));
            function errorHandler() {
                gitLogger.debug(`failed to clone repository from ${repo}`);
                reject(new Error(`failed to clone repository from ${repo}
        ${errMsg}`));
            }
        });
    });
}
exports.clone = clone;
function prefixGitArgs() {
    return process.platform === 'win32' ? ['-c', 'core.longpaths=true'] : [];
}
function spawnGit(args) {
    gitLogger.debug(`executing git with args ${args}`);
    const fullArgs = prefixGitArgs().concat(args || []);
    return spawn('git', fullArgs);
}
//# sourceMappingURL=fetchResolution.js.map