"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const path = require("path");
/**
 * Resolves a 'hosted' package hosted on 'github'.
 */
function resolveGithub(spec, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const ghSpec = parseGithubSpec(spec);
        // the ref should be a commit sha. Otherwise it would't be unique
        // and couldn't be saved in a machine store
        ghSpec.ref = yield resolveRef(ghSpec);
        return {
            id: path.join('github.com', ghSpec.owner, ghSpec.repo, ghSpec.ref),
            tarball: `https://codeload.github.com/${ghSpec.owner}/${ghSpec.repo}/tar.gz/${ghSpec.ref}`
        };
        function resolveRef(spec) {
            return __awaiter(this, void 0, void 0, function* () {
                const url = [
                    'https://api.github.com/repos',
                    spec.owner,
                    spec.repo,
                    'commits',
                    spec.ref
                ].join('/');
                const body = yield opts.got.getJSON(url);
                return body.sha;
            });
        }
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = resolveGithub;
const PARSE_GITHUB_RE = /^github:([^\/]+)\/([^#]+)(#(.+))?$/;
function parseGithubSpec(spec) {
    const m = PARSE_GITHUB_RE.exec(spec.hosted.shortcut);
    if (!m) {
        throw new Error('cannot parse: ' + spec.hosted.shortcut);
    }
    const owner = m[1];
    const repo = m[2];
    const ref = m[4] || 'HEAD';
    return { owner, repo, ref };
}
//# sourceMappingURL=github.js.map